var angular = global.angular || require('angular') && global.angular;
function safeDeepClone(circularValue, refs, obj) {
  var copy;

  // object is a false or empty value, or otherwise not an object
  if (!obj || 'object' !== typeof obj || obj instanceof Error || obj instanceof ArrayBuffer || obj instanceof Blob || obj instanceof File) return obj;

  // Handle Date
  if (obj instanceof Date) {
    copy = new Date();
    copy.setTime(obj.getTime());
    return copy;
  }

  // Handle Array - or array-like items
  if (angular.isArray(obj)) {
    
    refs.push(obj);
    copy = [];
    for (var i = 0, len = obj.length; i < len; i++) {
      if (refs.indexOf(obj[i]) >= 0) {
        copy[i] = circularValue;
      } else {
        copy[i] = safeDeepClone(circularValue, refs, obj[i]);
      }
    }
    refs.pop();
    return copy;
  }

  // Handle Object
  refs.push(obj);

  // Bring a long prototype
  if (obj.constructor && obj.constructor !== Object) {
    copy = Object.create(obj.constructor.prototype);
  } else {
    copy = {};
  }

  for (var attr in obj) {
    if (obj.hasOwnProperty(attr) && attr !== '$$hashKey') {
      if (refs.indexOf(obj[attr]) >= 0) {
        copy[attr] = circularValue;
      } else {
        copy[attr] = safeDeepClone(circularValue, refs, obj[attr]);
      }
    }
  }
  refs.pop();
  return copy;
}

module.exports = safeDeepClone;
